// ----------------------------------------------------------
//  ""
// :  ..
// ()   , 2013
// ----------------------------------------------------------

#include "Convolution.h"

#include "Constants.h"
#include "Fft2D.h"
#include "MatrixHelper.h"
#include "ReferenceBeam.h"

namespace Holo
{

void Convolution::Reconstruct(const double** hologram, int sizeInPixel, 
						  double waveLength, double angle, double zDistance, double ccdPixelDistance, 
						  double**& intensity, double**& phase)
{
    double ccdLen = (sizeInPixel - 1) * ccdPixelDistance;
    
	alglib::complex** h = new alglib::complex*[sizeInPixel];
	for (int i = 0; i < sizeInPixel; i++) h[i] = new alglib::complex[sizeInPixel];

	alglib::complex** h2 = new alglib::complex*[sizeInPixel];
	for (int i = 0; i < sizeInPixel; i++) h2[i] = new alglib::complex[sizeInPixel];

	double** hologram2 = new double*[sizeInPixel];
	for (int i = 0; i < sizeInPixel; i++) hologram2[i] = new double[sizeInPixel];
    
	double avg2 = 0.0;
	for (int x = 0; x < sizeInPixel; x++)
	{
		for (int y = 0; y < sizeInPixel; y++)
		{
			double hh = hologram[x][y];
			if (x > 0 && y > 0) hh += hologram[x - 1][y - 1];
			if (x > 0) hh += hologram[x - 1][y];
			if (x > 0 && y < sizeInPixel - 1) hh += hologram[x - 1][y + 1];
			if (y > 0) hh += hologram[x][y - 1];
			if (y < sizeInPixel - 1) hh += hologram[x][y + 1];
			if (x < sizeInPixel - 1 && y < sizeInPixel - 1) hh += hologram[x + 1][y + 1];
			if (x < sizeInPixel - 1) hh += hologram[x + 1][y];
			if (x < sizeInPixel - 1 && y > 0) hh += hologram[x + 1][y - 1];
			hh /= 9;

			hologram2[x][y] = hologram[x][y] - hh;
			avg2 += hologram2[x][y];
		}
	}
	avg2 /= (sizeInPixel * sizeInPixel);

	ReferenceBeam refBeam(waveLength, sqrt(avg2));
    refBeam.GenerateField(ccdPixelDistance, sizeInPixel, angle, h);

	for (int x = 0; x < sizeInPixel; x++)
	{
		for (int y = 0; y < sizeInPixel; y++)
		{
			h[x][y].x *= hologram2[x][y];
			h[x][y].y *= hologram2[x][y];
		}
	}
	Fft2D::CalculateImplace(h, sizeInPixel, true);

	double wavenum = HOLO_2PI / waveLength;
	double Zeuclid = zDistance * zDistance;
	double half_ccdlen_inv = -0.5 * ccdLen;
	for (int x = 0; x < sizeInPixel; x++)
	{
		double ccdPx = half_ccdlen_inv + x * ccdPixelDistance;
		double Xeuclid = ccdPx * ccdPx;
		for (int y = 0; y < sizeInPixel; y++)
		{
			double ccdPy = half_ccdlen_inv + y * ccdPixelDistance;
			double Yeuclid = ccdPy * ccdPy;
			double dist = sqrt(Xeuclid + Yeuclid + Zeuclid);
			h2[x][y].x = sin(wavenum * dist) / (dist * waveLength);
			h2[x][y].y = cos(wavenum * dist) / (dist * waveLength);
		}
	}
	Fft2D::CalculateImplace(h2, sizeInPixel, true);

    for (int x = 0; x < sizeInPixel; x++)
    {
        for (int y = 0; y < sizeInPixel; y++)
        {
            double var1_real = h[x][y].x;
            double var1_imag = h[x][y].y;
			h[x][y].x = var1_real * h2[x][y].x - var1_imag * h2[x][y].y;
			h[x][y].y = var1_real * h2[x][y].y + var1_imag * h2[x][y].x;
        }
    }
	Fft2D::CalculateImplace(h, sizeInPixel, false);
	MatrixHelper::CyclicShift(h, sizeInPixel, sizeInPixel / 2, sizeInPixel / 2);

	for (int x = 0; x < sizeInPixel; x++)
	{
		for (int y = 0; y < sizeInPixel; y++)
		{
			if (intensity != 0) intensity[x][y] = h[x][y].x * h[x][y].x + h[x][y].y * h[x][y].y;
			phase[x][y] = h[x][y].y / h[x][y].x;
		}
	}

	RELEASE_MATRIX(h2, sizeInPixel);
	RELEASE_MATRIX(h, sizeInPixel);
	RELEASE_MATRIX(hologram2, sizeInPixel);
}

}