// ----------------------------------------------------------
//  ""
// :  ..
// ()   , 2013
// ----------------------------------------------------------

#include "DHI.h"

#include "Fresnel.h"
#include "Convolution.h"
#include "MatrixHelper.h"
#include "Constants.h"

namespace Holo
{

void DHI::Reconstruct(ReconstructionType type, const double** hologram1, const double** hologram2, int sizeInPixel, 
					  double waveLength, double angle, double zDistance, double ccdPixelDistance, 
					  double**& intensity1, double**& intensity2, double**& phaseDiff)
{
	double** phaseTmp = new double*[sizeInPixel];
	for (int i = 0; i < sizeInPixel; i++) phaseTmp[i] = new double[sizeInPixel];

	if (type == ReconstructionType::Fresnel)
	{
		Fresnel::Reconstruct((const double**)hologram1, sizeInPixel, waveLength, angle, zDistance, ccdPixelDistance, intensity1, phaseTmp);
		Fresnel::Reconstruct((const double**)hologram2, sizeInPixel, waveLength, angle, zDistance, ccdPixelDistance, intensity2, phaseDiff);
	}
	else if (type == ReconstructionType::Convolution)
	{
		Convolution::Reconstruct((const double**)hologram1, sizeInPixel, waveLength, angle, zDistance, ccdPixelDistance, intensity1, phaseTmp);
		Convolution::Reconstruct((const double**)hologram2, sizeInPixel, waveLength, angle, zDistance, ccdPixelDistance, intensity2, phaseDiff);
	}

	for (int x = 0; x < sizeInPixel; x++)
	{
		for (int y = 0; y < sizeInPixel; y++)
		{
			double t1 = (phaseTmp[x][y] - phaseDiff[x][y]);
			double t2 = (1.0 + phaseDiff[x][y] * phaseTmp[x][y]);
			phaseDiff[x][y] = atan2(t1, t2);
			if (phaseDiff[x][y] < 0) phaseDiff[x][y] += HOLO_2PI;
		}
	}

	RELEASE_MATRIX(phaseTmp, sizeInPixel);
}

}