// ----------------------------------------------------------
//  ""
// :  ..
// ()   , 2013
// ----------------------------------------------------------

#pragma once

#include "Timer.h"
#include <string>
#include <vector>
#include <list>
#include <ctime>
#include "pugixml.hpp"
#include "ImageLoader.h"

namespace Holo
{

class IExperimentListener;

class Experiment
{
public:
	struct ReconstructionParams
	{
		double waveLength;
		double angle;
		double zDistance;
		double ccdPixelDistance;
		ReconstructionParams() : waveLength(0.000628), angle(0), zDistance(300.0), ccdPixelDistance(0.005){}
		ReconstructionParams(const ReconstructionParams& rp)
		{
			waveLength = rp.waveLength;
			angle = rp.angle;
			zDistance = rp.zDistance;
			ccdPixelDistance = rp.ccdPixelDistance;
		}
	};

	struct OutputParams
	{
		bool normalize;
		double brigthness;
		OutputParams() : normalize(true), brigthness(1.0) {}
		OutputParams(const OutputParams& op)
		{
			normalize = op.normalize;
			brigthness = op.brigthness;
		}
	};

	enum ExperimentType
	{
		Reconstruction,
		DHI
	};

	enum ReconstructionType
	{
		Fresnel,
		Convolution
	};

	Experiment();
	~Experiment(){}

	void Undertake(const std::string& filename, IExperimentListener* listener = 0);

private:
	ExperimentType m_type;
	ReconstructionType m_recType;
	std::string m_hologramPath;
	std::string m_hologram2Path;
	ImageLoader::ClipType m_clipType;
	int m_preferredSize;

	Timer m_timer;
	Timer m_timerFull;

	std::vector<ReconstructionParams> m_reconstructionParams;
	OutputParams m_outputParams;

	void Step(IExperimentListener* listener, int stepIndex, const double** hologram, int sizeInPixel, 
			  double**& intensity, double**& phase, ReconstructionParams& recParams, OutputParams& outputParams,
			  const std::string& resultFolder);

	void StepDHI(IExperimentListener* listener, int stepIndex, const double** hologram1, const double** hologram2, int sizeInPixel, 
				 double**& intensity1, double**& intensity2, double**& phaseDiff, double**& phaseUnwrapped, ReconstructionParams& recParams, OutputParams& outputParams,
				 const std::string& resultFolder);

	std::list<double> GetValues(const pugi::xml_node& node);

	bool Load(const std::string& filename);
	void SaveResults(const std::string& filename, double fullTime, double avgTime);

	void LoadImage(IExperimentListener* listener, const std::string& filename, double**& data, int& size);

	void SaveCsv(const std::string& filename, const double** data, int size);
};

class IExperimentListener
{
public:
	virtual ~IExperimentListener(){}
	virtual void OnStart(const std::string& experimentFilename, const tm& startDateTime, Experiment::ExperimentType type, Experiment::ReconstructionType recType) = 0;
	virtual void OnFinish() = 0;
	virtual void OnStep(int step, int stepsCount, double time, double avgTime) = 0;
	virtual void OnError(const std::string& errorDescr) = 0;
};

}