// ----------------------------------------------------------
//  ""
// :  ..
// ()   , 2013
// ----------------------------------------------------------

#include "Fresnel.h"

#include "Constants.h"
#include "Fft2D.h"
#include "MatrixHelper.h"
#include "ReferenceBeam.h"

namespace Holo
{

void Fresnel::Reconstruct(const double** hologram, int sizeInPixel, 
						  double waveLength, double angle, double zDistance, double ccdPixelDistance, 
						  double**& intensity, double**& phase)
{
    double ccdLen = (sizeInPixel - 1) * ccdPixelDistance;
    
	alglib::complex** h = new alglib::complex*[sizeInPixel];
	for (int i = 0; i < sizeInPixel; i++) h[i] = new alglib::complex[sizeInPixel];

	double** hologram2 = new double*[sizeInPixel];
	for (int i = 0; i < sizeInPixel; i++) hologram2[i] = new double[sizeInPixel];
    
	double avg2 = 0.0;
	for (int x = 0; x < sizeInPixel; x++)
	{
		for (int y = 0; y < sizeInPixel; y++)
		{
			double hh = hologram[x][y];
			if (x > 0 && y > 0) hh += hologram[x - 1][y - 1];
			if (x > 0) hh += hologram[x - 1][y];
			if (x > 0 && y < sizeInPixel - 1) hh += hologram[x - 1][y + 1];
			if (y > 0) hh += hologram[x][y - 1];
			if (y < sizeInPixel - 1) hh += hologram[x][y + 1];
			if (x < sizeInPixel - 1 && y < sizeInPixel - 1) hh += hologram[x + 1][y + 1];
			if (x < sizeInPixel - 1) hh += hologram[x + 1][y];
			if (x < sizeInPixel - 1 && y > 0) hh += hologram[x + 1][y - 1];
			hh /= 9;

			hologram2[x][y] = hologram[x][y] - hh;
			avg2 += hologram2[x][y];
		}
	}
	avg2 /= (sizeInPixel * sizeInPixel);

	ReferenceBeam refBeam(waveLength, sqrt(avg2));
    refBeam.GenerateField(ccdPixelDistance, sizeInPixel, angle, h);

    double factor1 = -HOLO_PI / (waveLength * zDistance);
	double half_ccdlen_inv = -0.5 * ccdLen;
    for (int x = 0; x < sizeInPixel; x++)
    {
		double ccdPx = half_ccdlen_inv + x * ccdPixelDistance;
        double Xeuclid = ccdPx * ccdPx;
        for (int y = 0; y < sizeInPixel; y++)
        {
			double ccdPy = half_ccdlen_inv + y * ccdPixelDistance;
            double Yeuclid = ccdPy * ccdPy;
            double euclid = Xeuclid + Yeuclid;
            double var1_real = cos(factor1 * euclid);
            double var1_imag = sin(factor1 * euclid);
			double ref_real = h[x][y].x;
			double ref_imag = h[x][y].y;
            
			h[x][y].x = hologram2[x][y] * (ref_real * var1_real - ref_imag * var1_imag);
			h[x][y].y = hologram2[x][y] * (ref_real * var1_imag + ref_imag * var1_real);
        }
    }

    Fft2D::CalculateImplace(h, sizeInPixel, false);
    MatrixHelper::CyclicShift(h, sizeInPixel, sizeInPixel / 2, sizeInPixel / 2);

    double factor2 = -HOLO_PI * waveLength * zDistance;
	double wavenum = HOLO_2PI / waveLength;
    double factor3 = -wavenum * zDistance;
	double factor4 = 1.0 / (waveLength * zDistance);
    for (int x = 0; x < sizeInPixel; x++)
    {
        double Xdist = ((x - (sizeInPixel - 1) / 2) / (ccdLen + ccdPixelDistance));
        Xdist *= Xdist;
        for (int y = 0; y < sizeInPixel; y++)
        {
            double Ydist = ((y - (sizeInPixel - 1) / 2) / (ccdLen + ccdPixelDistance));
            Ydist *= Ydist;
            double dist = Xdist + Ydist;
            double var1_real = cos(factor2 * dist);
            double var1_imag = sin(factor2 * dist);
			double var4_real = (sin(factor3) * var1_real - cos(factor3) * var1_imag) * factor4;
            double var4_imag = (sin(factor3) * var1_imag + cos(factor3) * var1_real) * factor4;
            double ccd_real = var4_real * h[x][y].x - var4_imag * h[x][y].y;
            double ccd_imag = var4_real * h[x][y].y + var4_imag * h[x][y].x;

			if (intensity != 0) intensity[x][y] = ccd_real * ccd_real + ccd_imag * ccd_imag;
            phase[x][y] = atan2(ccd_imag, ccd_real);
        }
    }

	RELEASE_MATRIX(h, sizeInPixel);
	RELEASE_MATRIX(hologram2, sizeInPixel);
}

}