// ----------------------------------------------------------
//  ""
// :  ..
// ()   , 2013
// ----------------------------------------------------------

#include <iostream>

#include "Experiment.h"

using namespace std;

class ExperimentListener : public Holo::IExperimentListener
{
public:
	virtual ~ExperimentListener(){}
	virtual void OnStart(const std::string& experimentFilename, const tm& startDateTime, 
						 Holo::Experiment::ExperimentType type, Holo::Experiment::ReconstructionType recType)
	{
		if (recType == Holo::Experiment::ReconstructionType::Fresnel) cout << "Fresnel";
		else if (recType == Holo::Experiment::ReconstructionType::Convolution) cout << "Convolution";
		else cout << "Undefined";

		if (type == Holo::Experiment::Reconstruction)
		{
			cout << " reconstruction ";
		}
		else if (type == Holo::Experiment::DHI)
		{
			cout << " DHI ";
		}
		else
			cout << "Undefined ";
		cout << "experiment (" << experimentFilename << ") is starting..." << endl;
	}

	virtual void OnFinish()
	{
		cout << "The experiment has finished" << endl;
	}

	virtual void OnStep(int step, int stepsCount, double time, double avgTime)
	{
		double t = avgTime * (stepsCount - step);
		int minutes = (int)(t / 60);
		int seconds = (int)(t - (double)minutes * 60.0);
		cout << "Step " << step << "/" << stepsCount << " completed (" << time << "s.). Remained ~";
		if (minutes == 0) cout << (int)t << "s.\n";
		else cout << minutes << "m. " << seconds << "s.\n";
	}
	
	void OnError(const std::string& errorDescr)
	{
		cerr << "Error: " << errorDescr << endl;
	}
};

void main(int argc, char** argv)
{
	Holo::Timer::Init();

	std::string experimentFilename = "experiment.xml";
	if (argc > 1)
	{
		experimentFilename = argv[1];
	}

	Holo::Experiment exp1;
	exp1.Undertake(experimentFilename, &ExperimentListener());
}